@@ -69,10 +69,12 @@ class Notifier(object):
 
     def msg(self, body, edit_cb, switch_cb):
         self._notify = pynotify.Notification(self.summary, body, self._icon, self._attach)
-        #translators: this is edit activity action in the notifier bubble
-        self._notify.add_action("edit", _("Edit"), edit_cb)
-        #translators: this is switch activity action in the notifier bubble
-        self._notify.add_action("switch", _("Switch"), switch_cb)
+
+        if "actions" in pynotify.get_server_caps():
+            #translators: this is edit activity action in the notifier bubble
+            self._notify.add_action("edit", _("Edit"), edit_cb)
+            #translators: this is switch activity action in the notifier bubble
+            self._notify.add_action("switch", _("Switch"), switch_cb)
         self._notify.show()
 
 
@@ -109,6 +111,7 @@ class PanelButton(gtk.ToggleButton):
             else:
                 label = "%s %s" % (self.activity, self.duration)
         
+        label = escape_pango(label)
         label = '<span gravity=\"south\">' + label + '</span>'
         self.label.set_markup(label)
 
@@ -199,6 +202,8 @@ class HamsterApplet(object):
         
         self.applet = applet
         self.applet.set_applet_flags (gnomeapplet.EXPAND_MINOR);
+        
+        self.notify_interval = None
 
         self.preferences_editor = None
         self.applet.about = None
@@ -229,6 +234,7 @@ class HamsterApplet(object):
         
         edit_cell = gtk.CellRendererPixbuf()
         edit_cell.set_property("stock_id", "gtk-edit")
+        edit_cell.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
         self.edit_column = gtk.TreeViewColumn("", edit_cell)
         self.treeview.append_column(self.edit_column)
 
@@ -340,6 +346,9 @@ class HamsterApplet(object):
         completion = gtk.EntryCompletion()
         completion.set_model(self.activities)
 
+        completion.set_text_column(2)
+        completion.clear()
+
         activity_cell = gtk.CellRendererText()
         completion.pack_start(activity_cell, True)
         completion.add_attribute(activity_cell, 'text', 0)
@@ -348,16 +357,6 @@ class HamsterApplet(object):
         category_cell = CategoryCell()  
         completion.pack_start(category_cell, False)
         completion.add_attribute(category_cell, 'text', 1)
-
-
-        def match_func(completion, key, iter):
-            model = completion.get_model()
-            text = model.get_value(iter, 2)
-            if text and text.startswith(key):
-                return True
-            return False
-
-        completion.set_match_func(match_func)
         completion.set_minimum_key_length(1)
         completion.set_inline_completion(True)
 
@@ -404,7 +403,12 @@ class HamsterApplet(object):
             # if we have date change - let's finish previous task and start a new one
             if prev_date and prev_date != self.today: 
                 storage.touch_fact(self.last_activity)
-                storage.add_fact(self.last_activity['name'])
+                activity_name = self.last_activity['name']
+                if self.last_activity['category']:
+                    activity_name += "@%s" % self.last_activity['category']
+                if self.last_activity['description']:
+                    activity_name += ",%s" % self.last_activity['description']
+                storage.add_fact(activity_name)
             elif self.button.get_active(): # otherwise if we the day view is visible - update day's durations
                 self.load_day()
                     
@@ -473,13 +477,16 @@ class HamsterApplet(object):
         #first populate the autocomplete - contains all entries in lowercase
         self.activities.clear()
         all_activities = storage.get_autocomplete_activities()
+
         for activity in all_activities:
-            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            activity_category = activity['name']
+            if activity['category']:
+                activity_category += "@%s" % activity['category']
+                
             self.activities.append([activity['name'],
                                     activity['category'],
                                     activity_category])
 
-
         #now populate the menu - contains only categorized entries
         store = self.activity_list.get_model()
         store.clear()
@@ -488,11 +495,15 @@ class HamsterApplet(object):
         categorized_activities = storage.get_sorted_activities()
 
         for activity in categorized_activities:
-            activity_category = "%s@%s" % (activity['name'], activity['category'])
+            activity_category = activity['name']
+            if activity['category']:
+                activity_category += "@%s" % activity['category']
+
             item = store.append([activity['name'],
                                  activity['category'],
                                  activity_category])
 
+
         # finally add TODO tasks from evolution to both lists
         tasks = hamster.eds.get_eds_tasks()
         for activity in tasks:
@@ -596,25 +607,52 @@ class HamsterApplet(object):
         
         if activity_name == "":
             return
-        
-        storage.add_fact(activity_name)
-        dispatcher.dispatch('panel_visible', False)
+        self.add_fact(activity_name)
 
     """listview events"""
-    def on_todays_keys(self, tree, event_key):
-        if (event_key.keyval == gtk.keysyms.Delete):
+    def on_todays_keys(self, tree, event):
+        if (event.keyval == gtk.keysyms.Delete):
             self.delete_selected()
             return True
+        elif (event.keyval == gtk.keysyms.e  \
+              and event.state & gtk.gdk.CONTROL_MASK):
+            self._open_edit_activity()
+            return True
+            
         return False
     
-    def on_today_row_activated(self, tree, path, column):
-        selection = tree.get_selection()
+    def _open_edit_activity(self):
+        """opens activity editor for selected row"""
+        selection = self.treeview.get_selection()
         (model, iter) = selection.get_selected()
-        activity_name = model[iter][1]
-        if activity_name:
-            storage.add_fact(activity_name)
-            dispatcher.dispatch('panel_visible', False)        
+        fact_id = model[iter][0]
+            
+        custom_fact = CustomFactController(None, fact_id)
+        custom_fact.show()        
+    
+    def on_today_row_activated(self, tree, path, column):
+        if column == self.edit_column:
+            self._open_edit_activity()
+        else:
+            selection = tree.get_selection()
+            (model, iter) = selection.get_selected()
+            activity_name = model[iter][1].decode('utf8', 'replace')
+            if activity_name:
+                description = model[iter][5]
+                if description:
+                    description = description.decode('utf8', 'replace')
+                    activity_name = "%s, %s" % (activity_name, description)
+                    
+                self.add_fact(activity_name)
+    
+    def add_fact(self, activity_name):
+        if self.last_activity and self.last_activity['start_time'] != datetime.date.today():
+            # very seldom thing but we have been caught right after midnight
+            self.refresh_hamster()
+        storage.add_fact(activity_name)
+        dispatcher.dispatch('panel_visible', False)        
         
+
         
     def on_windows_keys(self, tree, event_key):
         if (event_key.keyval == gtk.keysyms.Escape
